import type { APIContext, MiddlewareHandler } from '../../@types/astro';
/**
 * Utility function that is in charge of calling the middleware
 *
 * It accepts a `R` generic, which usually the `Response` returned.
 * It is a generic because endpoints can return a different response.
 *
 * When calling a middleware, we provide a `resolve` function, this function might or
 * might not be called. Because of that, we use a `Promise.race` to understand which
 * promise is resolved first.
 *
 * If `resolve` is called first, we resolve the `responseFunction` and we pass that response
 * as resolved value to `resolve`. Finally, we resolve the middleware.
 * This logic covers examples like:
 *
 * ```js
 * const onRequest = async (context, resolve) => {
 *   const response = await resolve(context);
 *   return response;
 * }
 * ```
 *
 * If the middleware is called first, we return the response without fancy logic. This covers cases like:
 *
 * ```js
 * const onRequest = async (context, _) => {
 *   context.locals = "foo";
 * }
 * ```
 *
 * @param onRequest The function called which accepts a `context` and a `resolve` function
 * @param apiContext The API context
 * @param responseFunction A callback function that should return a promise with the response
 */
export declare function callMiddleware<R>(onRequest: MiddlewareHandler<R>, apiContext: APIContext, responseFunction: () => Promise<R>): Promise<Response | R>;
